@@ -20,10 +20,10 @@ Examples:
   from uncertainties.umath import *  # sin(), etc.
 
   # Mathematical operations:
-  x = ufloat((0.20, 0.01))  # x = 0.20+/-0.01
-  x = ufloat("0.20+/-0.01")  # Other representation
-  x = ufloat("0.20(1)")  # Other representation
-  x = ufloat("0.20")  # Implicit uncertainty of +/-1 on the last digit
+  x = ufloat(0.20, 0.01)  # x = 0.20+/-0.01
+  x = ufloat_fromstr("0.20+/-0.01")  # Other representation
+  x = ufloat_fromstr("0.20(1)")  # Other representation
+  x = ufloat_fromstr("0.20")  # Implicit uncertainty of +/-1 on the last digit
   print x**2  # Square: prints "0.04+/-0.004"
   print sin(x**2)  # Prints "0.0399...+/-0.00399..."
 
@@ -33,16 +33,16 @@ Examples:
   square = x**2  # Square
   print square  # Prints "0.04+/-0.004"  
   print square.nominal_value  # Prints "0.04"
-  print square.std_dev()  # Prints "0.004..."
+  print square.std_dev  # Prints "0.004..."
 
   print square.derivatives[x]  # Partial derivative: 0.4 (= 2*0.20)
 
   # Correlations:
-  u = ufloat((1, 0.05), "u variable")  # Tag
-  v = ufloat((10, 0.1), "v variable")
+  u = ufloat(1, 0.05, "u variable")  # Tag
+  v = ufloat(10, 0.1, "v variable")
   sum_value = u+v
   
-  u.set_std_dev(0.1)  # Standard deviations can be updated on the fly
+  u.std_dev = 0.1  # Standard deviations can be updated on the fly
   print sum_value - u - v  # Prints "0.0" (exact result)
 
   # List of all sources of error:
@@ -75,7 +75,7 @@ on numbers with uncertainties by using their generalization from the
 uncertainties.umath module:
 
   from uncertainties.umath import sin
-  print sin(ufloat("1+/-0.01"))  # 0.841...+/-0.005...
+  print sin(ufloat_fromstr("1+/-0.01"))  # 0.841...+/-0.005...
   print sin(1)  # umath.sin() also works on floats, exactly like math.sin()
 
 Logical operations (>, ==, etc.) are also supported.
@@ -83,7 +83,7 @@ Logical operations (>, ==, etc.) are also supported.
 Basic operations on NumPy arrays or matrices of numbers with
 uncertainties can be performed:
 
-  2*numpy.array([ufloat((1, 0.01)), ufloat((2, 0.1))])
+  2*numpy.array([ufloat(1, 0.01), ufloat(2, 0.1)])
 
 More complex operations on NumPy arrays can be performed through the
 dedicated uncertainties.unumpy sub-module (see its documentation).
@@ -139,7 +139,7 @@ average value.
 uncertainties, it can be useful to access the nominal value and
 uncertainty of all numbers in a uniform manner:
 
-  x = ufloat("3+/-0.1")
+  x = ufloat_fromstr("3+/-0.1")
   print nominal_value(x)  # Prints 3
   print std_dev(x)  # Prints 0.1
   print nominal_value(3)  # Prints 3: nominal_value works on floats
@@ -194,8 +194,8 @@ Example:
 
 but
 
-  x = ufloat((3.14, 0.01))
-  y = ufloat((3.14, 0.01))
+  x = ufloat(3.14, 0.01)
+  y = ufloat(3.14, 0.01)
 
 is not such that x == y, since x and y are independent random
 variables that almost never give the same value.  However, x == x
@@ -235,7 +235,7 @@ import copy
 import warnings
 
 # Numerical version:
-__version_info__ = (1, 9, 1)
+__version_info__ = (2, 1)
 __version__ = '.'.join(map(str, __version_info__))
 
 __author__ = 'Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>'
@@ -301,8 +301,11 @@ def deprecation(message):
     Warns the user with the given message, by issuing a
     DeprecationWarning.
     '''
-    warnings.warn(message, DeprecationWarning, stacklevel=2)
 
+    # stacklevel = 3 points to the original user call (not to the
+    # function from this module that called deprecation()).
+    # DeprecationWarning is ignored by default: not used.
+    warnings.warn(message, stacklevel=3)
 
 ###############################################################################
 
@@ -756,7 +759,7 @@ def _eq_on_aff_funcs(self, y_with_uncert):
     difference = self - y_with_uncert
     # Only an exact zero difference means that self and y are
     # equal numerically:
-    return not(difference._nominal_value or difference.std_dev())
+    return not(difference._nominal_value or difference.std_dev)
 
 def _ne_on_aff_funcs(self, y_with_uncert):
     """
@@ -800,6 +803,18 @@ def _le_on_aff_funcs(self, y_with_uncert):
 
 ########################################
 
+class CallableStdDev(float):
+    '''
+    Class for standard deviation results, which used to be
+    callable. Provided for compatibility with old code. Issues an
+    obsolescence warning upon call.
+    '''
+    
+    def __call__ (self):
+        deprecation('Obsolete: the std_dev attribute should not be called'
+                    ' anymore: use .std_dev instead of .std_dev().')
+        return self
+        
 class AffineScalarFunc(object):
     """
     Affine functions that support basic mathematical operations
@@ -960,7 +975,8 @@ class AffineScalarFunc(object):
         """
         Individual components of the standard deviation of the affine
         function (in absolute value), returned as a dictionary with
-        Variable objects as keys.
+        Variable objects as keys. The returned variables are the
+        independent variables that the affine function depends on.
 
         This method assumes that the derivatives contained in the
         object take scalar values (and are not a tuple, like what
@@ -974,7 +990,8 @@ class AffineScalarFunc(object):
             error_components[variable] = abs(derivative*variable._std_dev)
 
         return error_components
-
+    
+    @property
     def std_dev(self):
         """
         Standard deviation of the affine function.
@@ -982,7 +999,7 @@ class AffineScalarFunc(object):
         This method assumes that the function returns scalar results.
 
         This returned standard deviation depends on the current
-        standard deviations [std_dev()] of the variables (Variable
+        standard deviations [std_dev] of the variables (Variable
         objects) involved.
         """
         #! It would be possible to not allow the user to update the
@@ -991,8 +1008,8 @@ class AffineScalarFunc(object):
         #std_dev value (in fact, many intermediate AffineScalarFunc do
         #not need to have their std_dev calculated: only the final
         #AffineScalarFunc returned to the user does).
-        return sqrt(sum(
-            delta**2 for delta in self.error_components().itervalues()))
+        return CallableStdDev(sqrt(sum(
+            delta**2 for delta in self.error_components().itervalues())))
 
     def _general_representation(self, to_string):
         """
@@ -1003,7 +1020,7 @@ class AffineScalarFunc(object):
         to_string() is typically repr() or str().
         """
 
-        (nominal_value, std_dev) = (self._nominal_value, self.std_dev())
+        (nominal_value, std_dev) = (self._nominal_value, self.std_dev)
 
         # String representation:
 
@@ -1032,7 +1049,7 @@ class AffineScalarFunc(object):
         try:
             # The ._nominal_value is a float: there is no integer division,
             # here:
-            return (value - self._nominal_value) / self.std_dev()
+            return (value - self._nominal_value) / self.std_dev
         except ZeroDivisionError:
             raise ValueError("The standard deviation is zero:"
                              " undefined result.")
@@ -1106,7 +1123,6 @@ class AffineScalarFunc(object):
                 all_slots.update(slot_names)
 
         # The slot values are stored:
-        # !! Python 2.7+: {name: getattr...}:
         for name in all_slots:
             try:
                 # !! It might happen that '__dict__' is itself a slot
@@ -1304,39 +1320,38 @@ class Variable(AffineScalarFunc):
         # takes much more memory.  Thus, this implementation chooses
         # more cycles and a smaller memory footprint instead of no
         # cycles and a larger memory footprint.
-
-        # ! Using AffineScalarFunc instead of super() results only in
-        # a 3 % speed loss (Python 2.6, Mac OS X):
         super(Variable, self).__init__(value, {self: 1.})
 
-        # We force the error to be float-like.  Since it is considered
-        # as a Gaussian standard deviation, it is semantically
-        # positive (even though there would be no problem defining it
-        # as a sigma, where sigma can be negative and still define a
-        # Gaussian):
-
-        assert std_dev >= 0, "the error must be a positive number"
-        # Since AffineScalarFunc.std_dev is a property, we cannot do
-        # "self.std_dev = ...":
-        self._std_dev = std_dev
+        self.std_dev = std_dev  # Assignment through a Python property
         
         self.tag = tag
 
+    # !! In Python 2.6+, the std_dev property would be more simply
+    # implemented with @property(getter), then @std_dev.setter(setter).
+
     # Standard deviations can be modified (this is a feature).
     # AffineScalarFunc objects that depend on the Variable have their
-    # std_dev() automatically modified (recalculated with the new
+    # std_dev automatically modified (recalculated with the new
     # std_dev of their Variables):
-    def set_std_dev(self, value):
-        """
-        Updates the standard deviation of the variable to a new value.
-        """
+    def _set_std_dev(self, std_dev):
+    
+        # We force the error to be float-like.  Since it is considered
+        # as a standard deviation, it must be positive:
+        assert std_dev >= 0, "the error must be a positive number"
+
+        self._std_dev = CallableStdDev(std_dev)
+    
+    def _get_std_dev(self):
+        return self._std_dev
+        
+    std_dev = property(_get_std_dev, _set_std_dev)
+    
+    # Support for legacy method:
+    def set_std_dev(self, value):  # Obsolete
+        deprecation('Obsolete: instead of set_std_dev(), please use'
+                    ' .std_dev = ...')
+        self.std_dev = value
         
-        # A zero variance is accepted.  Thus, it is possible to
-        # conveniently use infinitely precise variables, for instance
-        # to study special cases.
-
-        self._std_dev = value
-
     # The following method is overridden so that we can represent the tag:
     def _general_representation(self, to_string):
         """
@@ -1377,7 +1392,7 @@ class Variable(AffineScalarFunc):
         #! The following assumes that the arguments to Variable are
         # *not* copied upon construction, since __copy__ is not supposed
         # to copy "inside" information:
-        return Variable(self.nominal_value, self.std_dev(), self.tag)
+        return Variable(self.nominal_value, self.std_dev, self.tag)
 
     def __deepcopy__(self, memo):
         """
@@ -1421,7 +1436,7 @@ def std_dev(x):
     numbers, when only some of them generally carry an uncertainty.
     """
 
-    return x.std_dev() if isinstance(x, AffineScalarFunc) else 0.
+    return x.std_dev if isinstance(x, AffineScalarFunc) else 0.
 
 def covariance_matrix(nums_with_uncert):
     """
@@ -1443,11 +1458,13 @@ def covariance_matrix(nums_with_uncert):
     # See PSI.411 in EOL's notes.
 
     covariance_matrix = []
+    # ! In Python 2.6+, this could be replaced by enumerate(..., 1),
+    # along with updating places where i1 is used (i1+1 => i1).
     for (i1, expr1) in enumerate(nums_with_uncert):
         derivatives1 = expr1.derivatives  # Optimization
         vars1 = set(derivatives1)
         coefs_expr1 = []
-        for (i2, expr2) in enumerate(nums_with_uncert[:i1+1]):
+        for expr2 in nums_with_uncert[:i1+1]:
             derivatives2 = expr2.derivatives  # Optimization
             coef = 0.
             for var in vars1.intersection(derivatives2):
@@ -1553,9 +1570,9 @@ def parse_error_in_parentheses(representation):
 
     
 # The following function is not exposed because it can in effect be
-# obtained by doing x = ufloat(representation) and
-# x.nominal_value and x.std_dev():
-def str_to_number_with_uncert(representation):
+# obtained by doing x = ufloat_fromstr(representation) and reading
+# x.nominal_value and x.std_dev:
+def _str_to_number_with_uncert(representation):
     """
     Given a string that represents a number with uncertainty, returns the
     nominal value and the uncertainty.
@@ -1587,26 +1604,15 @@ def str_to_number_with_uncert(representation):
 
     return parsed_value
 
-def ufloat(representation, tag=None):
+def ufloat_fromstr(representation, tag=None):
     """
-    Returns a random variable (Variable object).
-
-    Converts the representation of a number into a number with
-    uncertainty (a random variable, defined by a nominal value and
-    a standard deviation).
-
-    The representation can be a (value, standard deviation) sequence,
-    or a string.
-
-    Strings of the form '12.345+/-0.015', '12.345(15)', or '12.3' are
-    recognized (see full list below).  In the last case, an
-    uncertainty of +/-1 is assigned to the last digit.
-
-    'tag' is an optional string tag for the variable.  Variables
-    don't have to have distinct tags.  Tags are useful for tracing
-    what values (and errors) enter in a given result (through the
-    error_components() method).
-
+    Returns a new random variable (Variable object) from a string.
+    
+    Strings 'representation' of the form '12.345+/-0.015',
+    '12.345(15)', or '12.3' are recognized (see full list below).  In
+    the last case, an uncertainty of +/-1 is assigned to the last
+    digit.
+    
     Examples of valid string representations:
 
         -1.23(3.4)
@@ -1627,32 +1633,72 @@ def ufloat(representation, tag=None):
         169.1(15)
     """
 
-    # This function is somewhat optimized so as to help with the
-    # creation of lots of Variable objects (through unumpy.uarray, for
-    # instance).
-
-    # representations is "normalized" so as to be a valid sequence of
-    # 2 arguments for Variable().
-
-    #! Accepting strings and any kind of sequence slows down the code
-    # by about 5 %.  On the other hand, massive initializations of
-    # numbers with uncertainties are likely to be performed with
-    # unumpy.uarray, which does not support parsing from strings and
-    # thus does not have any overhead.
-
-    #! Different, in Python 3:
-    if isinstance(representation, basestring):
-        representation = str_to_number_with_uncert(representation)
-        
-    #! The tag is forced to be a string, so that the user does not
-    # create a Variable(2.5, 0.5) in order to represent 2.5 +/- 0.5.
-    # Forcing 'tag' to be a string prevents numerical uncertainties
-    # from being considered as tags, here:
-    if tag is not None:
-        #! 'unicode' is removed in Python3:
-        assert isinstance(tag, (str, unicode)), "The tag can only be a string."
-
     #! The special ** syntax is for Python 2.5 and before (Python 2.6+
     # understands tag=tag):
-    return Variable(*representation, **{'tag': tag})
+    (nominal_value, std_dev) = _str_to_number_with_uncert(representation)
+    return ufloat(nominal_value, std_dev, tag)
+
+def _ufloat_obsolete(representation, tag=None):
+    '''
+    Legacy version of ufloat(). Will eventually be removed.
+
+    representation -- either a (nominal_value, std_dev) tuple, or a
+    string representation of a number with uncertainty, in a format
+    recognized by ufloat_fromstr().
+    '''
+    return (ufloat(representation[0], representation[1], tag)
+            if isinstance(representation, tuple)
+            else ufloat_fromstr(representation, tag))
+
+# The arguments are named for the new version, instead of bearing
+# names that are closer to their obsolete use (e.g., std_dev could be
+# instead std_dev_or_tag, since it can be the tag, in the obsolete
+# ufloat((3, 0.14), "pi") form). This has the advantage of allowing
+# new code to use keyword arguments as in ufloat(nominal_value=3,
+# std_dev=0.14), without breaking when the obsolete form is not
+# supported anymore.
+def ufloat(nominal_value, std_dev=None, tag=None):
+    """
+    Returns a new random variable (Variable object).
+    
+    The only non-obsolete use is:
+
+    - ufloat(nominal_value, std_dev),
+    - ufloat(nominal_value, std_dev, tag=...).
+
+    Other input parameters are temporarily supported:
+
+    - ufloat((nominal_value, std_dev)),
+    - ufloat((nominal_value, std_dev), tag),
+    - ufloat(str_representation),
+    - ufloat(str_representation, tag).
+
+    Valid string representations str_representation are listed in
+    the documentation for ufloat_fromstr().
+
+    'tag' is an optional string tag for the variable.  Variables
+    don't have to have distinct tags.  Tags are useful for tracing
+    what values (and errors) enter in a given result (through the
+    error_components() method).
+    """
+
+    try:
+        # Standard case:
+
+        #! The special ** syntax is for Python 2.5 and before (Python 2.6+
+        # understands tag=tag):
+        return Variable(nominal_value, std_dev, **{'tag': tag})
+    # Exception types raised by, respectively: tuple, string that
+    # cannot be converted through float(), and string that can be
+    # converted through float() (case of a number with no uncertainty):
+    except (TypeError, ValueError, AssertionError):
+        # Obsolete, two-argument call:
+        deprecation('Obsolete: either use ufloat(nominal_value, std_dev),'
+                    ' ufloat(nominal_value, std_dev, tag), or the'
+                    ' ufloat_fromstr() function, for string representations.')
+        return _ufloat_obsolete(nominal_value,  # Tuple or string
+                                # tag keyword used:
+                                tag if tag is not None
+                                # 2 positional arguments form:
+                                else std_dev)
 
