@@ -23,7 +23,7 @@ import sys
 # Local modules
 
 import uncertainties
-from uncertainties import ufloat, AffineScalarFunc
+from uncertainties import ufloat, AffineScalarFunc, ufloat_fromstr
 
 from uncertainties import __author__
 
@@ -193,6 +193,134 @@ def _compare_derivatives(func, numerical_derivatives,
                 break
 
 ###############################################################################
+
+def test_value_construction():
+    '''
+    Tests the various means of constructing a constant number with
+    uncertainty *without a string* (see test_str_input(), for this).
+    '''
+
+    ## Simple construction:
+    x = ufloat(3, 0.14)
+    assert x.nominal_value == 3
+    assert x.std_dev == 0.14
+    assert x.tag is None
+    
+    # ... with tag as positional argument:
+    x = ufloat(3, 0.14, 'pi')
+    assert x.nominal_value == 3
+    assert x.std_dev == 0.14
+    assert x.tag == 'pi'
+
+    # ... with tag keyword:
+    x = ufloat(3, 0.14, tag='pi')
+    assert x.nominal_value == 3
+    assert x.std_dev == 0.14
+    assert x.tag == 'pi'
+
+    ## Comparison with the obsolete tuple form:
+
+    # The following tuple is stored in a variable instead of being
+    # repeated in the calls below, so that the automatic code update
+    # does not replace ufloat((3, 0.14)) by ufloat(3, 14): the goal
+    # here is to make sure that the obsolete form gives the same
+    # result as the new form.
+    
+    representation = (3, 0.14)  # Obsolete representation
+    
+    x = ufloat(3, 0.14)
+    x2 = ufloat(representation)  # Obsolete
+    assert x.nominal_value == x2.nominal_value
+    assert x.std_dev == x2.std_dev
+    assert x.tag is None
+    assert x2.tag is None
+    
+    # With tag as positional argument:
+    x = ufloat(3, 0.14, "pi")
+    x2 = ufloat(representation, "pi")  # Obsolete
+    assert x.nominal_value == x2.nominal_value
+    assert x.std_dev == x2.std_dev
+    assert x.tag == 'pi'
+    assert x2.tag == 'pi'
+    
+    # With tag keyword:
+    x = ufloat(3, 0.14, tag="pi")
+    x2 = ufloat(representation, tag="pi")  # Obsolete
+    assert x.nominal_value == x2.nominal_value
+    assert x.std_dev == x2.std_dev
+    assert x.tag == 'pi'
+    assert x2.tag == 'pi'
+    
+def test_str_input():
+    "Input of numbers with uncertainties as a string"
+
+    # String representation, and numerical values:
+    tests = {
+        "-1.23(3.4)": (-1.23, 3.4),  # (Nominal value, error)
+        "-1.34(5)": (-1.34, 0.05),
+        "1(6)": (1, 6),
+        "3(4.2)": (3, 4.2),
+        "-9(2)": (-9, 2),
+        "1234567(1.2)": (1234567, 1.2),
+        "12.345(15)": (12.345, 0.015),
+        "-12.3456(78)e-6": (-12.3456e-6, 0.0078e-6),
+        "0.29": (0.29, 0.01),
+        "31.": (31, 1),
+        "-31.": (-31, 1),
+        # The following tests that the ufloat() routine does
+        # not consider '31' like the tuple ('3', '1'), which would
+        # make it expect two numbers (instead of 2 1-character
+        # strings):
+        "31": (31, 1),
+        "-3.1e10": (-3.1e10, 0.1e10),
+        "169.0(7)": (169, 0.7),
+        "-0.1+/-1": (-0.1, 1),
+        "-13e-2+/-1e2": (-13e-2, 1e2),
+        '-14.(15)': (-14, 15),
+        '-100.0(15)': (-100, 1.5),
+        '14.(15)': (14, 15)
+        }
+          
+    for (representation, values) in tests.iteritems():
+
+        # Without tag:
+        num = ufloat_fromstr(representation)
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag is None
+        
+        # With a tag as positional argument:
+        num = ufloat_fromstr(representation, 'test variable')
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag == 'test variable'
+
+        # With a tag as keyword argument:
+        num = ufloat_fromstr(representation, tag='test variable')
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag == 'test variable'
+        
+        ## Obsolete forms
+
+        num = ufloat(representation)  # Obsolete
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag is None
+        
+        # Call with a tag list argument:
+        num = ufloat(representation, 'test variable')  # Obsolete
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag == 'test variable'
+        
+        # Call with a tag keyword argument:
+        num = ufloat(representation, tag='test variable')  # Obsolete
+        assert _numbers_close(num.nominal_value, values[0])
+        assert _numbers_close(num.std_dev, values[1])
+        assert num.tag == 'test variable'
+
+###############################################################################
             
 # Test of correctness of the fixed (usually analytical) derivatives:
 def test_fixed_derivatives_basic_funcs():
@@ -233,7 +361,7 @@ def test_copy():
     "Standard copy module integration"
     import gc
     
-    x = ufloat((3, 0.1))
+    x = ufloat(3, 0.1)
     assert x == x
     
     y = copy.copy(x)
@@ -297,7 +425,7 @@ def test_pickling():
 
     import pickle
 
-    x = ufloat((2, 0.1))
+    x = ufloat(2, 0.1)
 
     x_unpickled = pickle.loads(pickle.dumps(x))
 
@@ -354,7 +482,7 @@ def test_int_div():
     "Integer division"
     # We perform all operations on floats, because derivatives can
     # otherwise be meaningless:
-    x = ufloat((3.9, 2))//2
+    x = ufloat(3.9, 2)//2
     assert x.nominal_value == 1.
     # All errors are supposed to be small, so the ufloat()
     # in x violates the assumption.  Therefore, the following is
@@ -368,15 +496,15 @@ def test_comparison_ops():
     
     # Operations on quantities equivalent to Python numbers must still
     # be correct:
-    a = ufloat((-3, 0))
-    b = ufloat((10, 0))
-    c = ufloat((10, 0))
+    a = ufloat(-3, 0)
+    b = ufloat(10, 0)
+    c = ufloat(10, 0)
     assert a < b
     assert a < 3
     assert 3 < b  # This is first given to int.__lt__()
     assert b == c
 
-    x = ufloat((3, 0.1))
+    x = ufloat(3, 0.1)
     
     # One constraint is that usual Python code for inequality testing
     # still work in a reasonable way (for instance, it is generally
@@ -398,12 +526,12 @@ def test_comparison_ops():
     assert x/2 == x/2
     # With uncorrelated result that have the same behavior (value and
     # standard error):
-    assert 2*ufloat((1, 0.1)) != ufloat((2, 0.2))    
+    assert 2*ufloat(1, 0.1) != ufloat(2, 0.2)    
     # Comparaison between 2 _different_ Variable objects
     # that are uncorrelated:
-    assert x != ufloat((3, 0.1))
+    assert x != ufloat(3, 0.1)
     
-    assert x != ufloat((3, 0.2))
+    assert x != ufloat(3, 0.2)
 
     # Comparison to other types should work:
     assert x != None  # Not comparable
@@ -486,26 +614,26 @@ def test_comparison_ops():
                                 % (x, op, y, correct_result))
 
     # With different numbers:
-    test_all_comparison_ops(ufloat((3, 0.1)),
-                            ufloat((-2, 0.1)))
-    test_all_comparison_ops(ufloat((0, 0)),  # Special number
-                            ufloat((1, 1)))
-    test_all_comparison_ops(ufloat((0, 0)),  # Special number
-                            ufloat((0, 0.1)))
+    test_all_comparison_ops(ufloat(3, 0.1),
+                            ufloat(-2, 0.1))
+    test_all_comparison_ops(ufloat(0, 0),  # Special number
+                            ufloat(1, 1))
+    test_all_comparison_ops(ufloat(0, 0),  # Special number
+                            ufloat(0, 0.1))
     # With identical numbers:
-    test_all_comparison_ops(ufloat((0, 0)),
-                            ufloat((0, 0)))
-    test_all_comparison_ops(ufloat((1, 1)),
-                            ufloat((1, 1)))
+    test_all_comparison_ops(ufloat(0, 0),
+                            ufloat(0, 0))
+    test_all_comparison_ops(ufloat(1, 1),
+                            ufloat(1, 1))
 
     
 def test_logic():
     "Boolean logic: __nonzero__, bool."
 
-    x = ufloat((3, 0))
-    y = ufloat((0, 0))
-    z = ufloat((0, 0.1))
-    t = ufloat((-1, 2))
+    x = ufloat(3, 0)
+    y = ufloat(0, 0)
+    z = ufloat(0, 0.1)
+    t = ufloat(-1, 2)
 
     assert bool(x) == True
     assert bool(y) == False
@@ -513,15 +641,17 @@ def test_logic():
     assert bool(t) == True  # Only infinitseimal neighborhood are used
 
 def test_obsolete():
-    'Tests obsolete functions'
-    x = ufloat((3, 0.1))
+    'Tests some obsolete functions'
+    x = ufloat(3, 0.1)
     x.set_std_dev(0.2)  # Obsolete function
-    assert x.std_dev() == 0.2  # Obsolete use with a call
+
+    x_std_dev = x.std_dev
+    assert x_std_dev() == 0.2  # Obsolete call
     
 def test_basic_access_to_data():
     "Access to data from Variable and AffineScalarFunc objects."
 
-    x = ufloat((3.14, 0.01), "x var")
+    x = ufloat(3.14, 0.01, "x var")
     assert x.tag == "x var"
     assert x.nominal_value == 3.14
     assert x.std_dev == 0.01
@@ -533,7 +663,7 @@ def test_basic_access_to_data():
     assert y.std_dev == 0.01
 
     # Details on the sources of error:
-    a = ufloat((-1, 0.001))
+    a = ufloat(-1, 0.001)
     y = 2*x + 3*x + 2 + a
     error_sources = y.error_components()
     assert len(error_sources) == 2  # 'a' and 'x'
@@ -570,8 +700,8 @@ def test_basic_access_to_data():
 def test_correlations():
     "Correlations between variables"
 
-    a = ufloat((1, 0))
-    x = ufloat((4, 0.1))
+    a = ufloat(1, 0)
+    x = ufloat(4, 0.1)
     y = x*2 + a
     # Correlations cancel "naive" additions of uncertainties:
     assert y.std_dev != 0
@@ -579,44 +709,6 @@ def test_correlations():
     assert normally_zero.nominal_value == 0
     assert normally_zero.std_dev == 0
 
-def test_str_input():
-
-    "Input of numbers with uncertainties as a string"
-
-    # String representation, and numerical values:
-    tests = {
-        "-1.23(3.4)": (-1.23, 3.4),  # (Nominal value, error)
-        "-1.34(5)": (-1.34, 0.05),
-        "1(6)": (1, 6),
-        "3(4.2)": (3, 4.2),
-        "-9(2)": (-9, 2),
-        "1234567(1.2)": (1234567, 1.2),
-        "12.345(15)": (12.345, 0.015),
-        "-12.3456(78)e-6": (-12.3456e-6, 0.0078e-6),
-        "0.29": (0.29, 0.01),
-        "31.": (31, 1),
-        "-31.": (-31, 1),
-        # The following tests that the ufloat() routine does
-        # not consider '31' like the tuple ('3', '1'), which would
-        # make it expect two numbers (instead of 2 1-character
-        # strings):
-        "31": (31, 1),
-        "-3.1e10": (-3.1e10, 0.1e10),
-        "169.0(7)": (169, 0.7),
-        "-0.1+/-1": (-0.1, 1),
-        "-13e-2+/-1e2": (-13e-2, 1e2),
-        '-14.(15)': (-14, 15),
-        '-100.0(15)': (-100, 1.5),
-        '14.(15)': (14, 15)
-        }
-          
-    for (representation, values) in tests.iteritems():
-        
-        num = ufloat(representation)
-
-        assert _numbers_close(num.nominal_value, values[0])
-        assert _numbers_close(num.std_dev, values[1])
-
     
 def test_no_coercion():
     """
@@ -625,7 +717,7 @@ def test_no_coercion():
     The coercion should be impossible, like for complex numbers.
     """
 
-    x = ufloat((4, 1))
+    x = ufloat(4, 1)
     try:
         assert float(x) == 4
     except TypeError:
@@ -653,7 +745,7 @@ def test_wrapped_func():
     # As a precaution, the wrapped function does not venture into
     # calculating f with uncertainties when one of the argument is not
     # a simple number, because this argument might contain variables:
-    angle = ufloat((0, 0.1))
+    angle = ufloat(0, 0.1)
 
     assert f_wrapped(angle, [angle, angle]) == NotImplemented
     assert f_wrapped(angle, my_list) == NotImplemented
@@ -664,7 +756,7 @@ def test_wrapped_func():
 def test_access_to_std_dev():
     "Uniform access to the standard deviation"
 
-    x = ufloat((1, 0.1))
+    x = ufloat(1, 0.1)
     y = 2*x
 
     # std_dev for Variable and AffineScalarFunc objects:
@@ -680,7 +772,7 @@ def test_access_to_std_dev():
 def test_covariances():
     "Covariance matrix"
 
-    x = ufloat((1, 0.1))
+    x = ufloat(1, 0.1)
     y = -2*x+10
     z = -3*x
     covs = uncertainties.covariance_matrix([x, y, z])
@@ -701,9 +793,9 @@ def test_power():
     and integral values of p.
     '''
 
-    zero = ufloat((0, 0))
-    one = ufloat((1, 0))
-    p = ufloat((0.3, 0.01))
+    zero = ufloat(0, 0)
+    one = ufloat(1, 0)
+    p = ufloat(0.3, 0.01)
 
     # assert 0**p == 0  # !!! Should pass
     # assert zero**p == 0  # !!! Should pass
@@ -759,12 +851,12 @@ def test_power():
         
     # Negative numbers with unceratinty can be exponentiated to an integral
     # power:
-    assert (ufloat((-1.1, 0.1))**-9).nominal_value == (-1.1)**-9
+    assert (ufloat(-1.1, 0.1)**-9).nominal_value == (-1.1)**-9
 
     # Case of numbers with no uncertainty: should give the same result
     # as numbers with uncertainties:
-    assert ufloat((-1, 0))**9 == (-1)**9
-    assert ufloat((-1.1, 0))**9 == (-1.1)**9
+    assert ufloat(-1, 0)**9 == (-1)**9
+    assert ufloat(-1.1, 0)**9 == (-1.1)**9
     
     # Negative numbers cannot be raised to a non-integral power, in
     # Python 2 (in Python 3, complex numbers are returned; this cannot
@@ -772,7 +864,7 @@ def test_power():
     # does not handle complex numbers):
     if sys.version_info < (3,):
         try:
-            ufloat((-1, 0))**9.1
+            ufloat(-1, 0)**9.1
         except Exception, err_ufloat:  # "as", for Python 2.6+
             pass
         else:
@@ -829,7 +921,7 @@ else:
     def test_numpy_comparison():
         "Comparison with a Numpy array."
 
-        x = ufloat((1, 0.1))
+        x = ufloat(1, 0.1)
         
         # Comparison with a different type:
         assert x != [x, x]
@@ -871,7 +963,7 @@ else:
         Test through the input of the (full) covariance matrix.
         """
 
-        u = uncertainties.ufloat((1, 0.1))
+        u = uncertainties.ufloat(1, 0.1)
         cov = uncertainties.covariance_matrix([u])
         # "1" is used instead of u.nominal_value because
         # u.nominal_value might return a float.  The idea is to force
@@ -884,8 +976,8 @@ else:
 
         # Covariances between output and input variables:
 
-        x = ufloat((1, 0.1))
-        y = ufloat((2, 0.3))
+        x = ufloat(1, 0.1)
+        y = ufloat(2, 0.3)
         z = -3*x+y
 
         covs = uncertainties.covariance_matrix([x, y, z])
@@ -918,8 +1010,8 @@ else:
 
         # ... as well as functional relations:
 
-        u = ufloat((1, 0.05))
-        v = ufloat((10, 0.1))
+        u = ufloat(1, 0.05)
+        v = ufloat(10, 0.1)
         sum_value = u+2*v
 
         # Covariance matrices:
@@ -948,8 +1040,8 @@ else:
         covariance matrix).
         '''
         
-        x = ufloat((1, 0.1))
-        y = ufloat((2, 0.3))
+        x = ufloat(1, 0.1)
+        y = ufloat(2, 0.3)
         z = -3*x+y
 
         cov_mat = uncertainties.covariance_matrix([x, y, z])
